const mqtt = require('mqtt')
const constants = require(".././constants.json")
console.log(constants.mqtt_url);

const client = mqtt.connect('mqtt://' + constants.mqtt_url)
const libSupport = require('./lib')
const execute = require('./execute')
const fs = require('fs')

const node_id = libSupport.makeid(10)
const local_repository = __dirname + "/local_repository/"
const host_url = "http://" + constants.master_address + ":" + constants.master_port
client.on('connect', function () {
    client.subscribe(node_id, function (err) {
        if (!err) {
            console.log("node listening to id", node_id);
            
        }
    })
})

client.on('message', function (topic, message) {
    
    message = JSON.parse(message)
    if (topic !== 'heartbeat') {
        let runtime = message.runtime
        let functionHash = message.functionHash
        let function_id = message.function_id
        
        if (message.type === "execute") {
            console.log("function_id", function_id);
            if (!fs.existsSync(local_repository + functionHash)) {
                libSupport.download(host_url + "/repository/" + functionHash, local_repository + functionHash).then(() => {
                    
                    if (runtime === "isolate")
                        execute.runIsolate(local_repository + functionHash).then(result => {
                            client.publish("response", JSON.stringify({
                                status: "success",
                                result,
                                function_id
                            }))
                        })
                    else if (runtime === "process")
                        execute.runProcess(local_repository + functionHash).then(result => {
                            client.publish("response", JSON.stringify({
                                status: "success",
                                result,
                                function_id
                            }))
                        })
                    else if (runtime === "container")
                        execute.runContainer(functionHash).then(result => {
                            client.publish("response", JSON.stringify({
                                status: "success",
                                result,
                                function_id
                            }))
                        })
                    else {
                        client.publish("response", JSON.stringify({ status: "unknown runtime" }))
                        return
                    }
                })
            } else {
                
                if (runtime === "isolate")
                    execute.runIsolate(local_repository + functionHash).then(result => {
                        client.publish("response", JSON.stringify({
                            status: "success",
                            result,
                            function_id
                        }))
                    })
                else if (runtime === "process")
                    execute.runProcess(local_repository + functionHash).then(result => {
                        client.publish("response", JSON.stringify({
                            status: "success",
                            result,
                            function_id
                        }))
                    })
                else if (runtime === "container")
                    execute.runContainer(functionHash).then(result => {
                        client.publish("response", JSON.stringify({
                            status: "success",
                            result,
                            function_id
                        }))
                    })
                else {
                    client.publish("response", JSON.stringify({ status: "unknown runtime" }))
                    return
                }
            }
            
        }
    }
        
})

function heartbeat() {
    client.publish("heartbeat", JSON.stringify({"address": node_id}))
}

setInterval(heartbeat, 1000);